var gulp = require('gulp')
  , ts = require('gulp-typescript')
  , tslint = require('gulp-tslint')
  , uglify = require('gulp-uglify')
  , rename = require('gulp-rename')
  , header = require('gulp-header')
  , sourcemaps = require('gulp-sourcemaps')
  , jasmine = require('gulp-jasmine')
  , del = require('del')
  , SpecReporter = require('jasmine-spec-reporter')
  , merge = require('merge2')
  , KarmaServer = require('karma').Server
  , pkg = require('./package.json')
  , tsProject = ts.createProject('tsconfig.json');



function banner () {
  return header(
    '/*\n' +
    ' * web-rpc\n' +
    ' * @version ' + pkg.version + '\n' +
    ' * @author ' + pkg.author + '\n' +
    ' * @copyright (c) 2015-2016 ' + pkg.author + '\n' +
    ' * @license MIT <https://github.com/web-rpc/web-rpc/blob/master/LICENSE>\n' +
    ' *\n' +
    ' * @overview ' + pkg.description + '\n' +
    ' */'
  );
}

gulp.task('lint', function(){
  return gulp.src('./src/**/*.ts')
    .pipe(tslint())
    .pipe(tslint.report('full'));
});


gulp.task('clean', function (done) {
  del(['dist'], done);
});

gulp.task('test:browser', function(done){
  new KarmaServer({
    configFile: process.cwd () + '/karma.conf.js'
  }, done).start();
});

gulp.task('test:server', function(){
  return gulp.src('test/server/**/*.spec.js')
    .pipe(jasmine({
      reporter: new SpecReporter()
    }));
});

gulp.task('compile', ['clean', 'lint'], function(){
  var tsResult = tsProject.src()
    .pipe(ts(tsProject));

  return merge([
    tsResult.dts.pipe(gulp.dest('./dist')),
    tsResult.js.pipe(banner()).pipe(gulp.dest('./dist'))
  ]);
});

gulp.task('test', ['test:browser', 'test:server']);

gulp.task('package', ['compile'], function(){
  return gulp.src('./dist/web-rpc.js')
    .pipe(sourcemaps.init())
    .pipe(uglify())
    .pipe(rename({
      extname: '.min.js'
    }))
    .pipe(banner())
    .pipe(sourcemaps.write())
    .pipe(gulp.dest('./dist'))
});
gulp.task('default', function(){
  gulp.watch(['./src/**/*.ts'], ['compile']);
  gulp.watch(['./test/server/**/*.spec.js'], ['test:server']);
  gulp.watch(['./test/client/**/*.spec.js'], ['test:client']);
});
